@@ -518,7 +518,7 @@ decode_picture(GstVaapiDecoderMpeg4 *decoder, const guint8 *buf, guint buf_size)
         parser_result = gst_mpeg4_parse_video_object_plane(vop_hdr, sprite_trajectory, vol_hdr, buf, buf_size);
         /* Need to skip this frame if VOP was not coded */
         if (GST_MPEG4_PARSER_OK == parser_result && !vop_hdr->coded)
-            return GST_VAAPI_DECODER_STATUS_DROP_FRAME;
+            return (GstVaapiDecoderStatus) GST_VAAPI_DECODER_STATUS_DROP_FRAME;
     }
 
     if (parser_result != GST_MPEG4_PARSER_OK) {
@@ -818,9 +818,6 @@ decode_packet(GstVaapiDecoderMpeg4 *decoder, GstMpeg4Packet packet)
     GstMpeg4Packet *tos = &packet;
     GstVaapiDecoderStatus status;
 
-    if (tos->size < 0)
-        return GST_VAAPI_DECODER_STATUS_ERROR_NO_DATA;
-
     // packet.size is the size from current marker to the next.
     if (tos->type == GST_MPEG4_VISUAL_OBJ_SEQ_START) {
         status = decode_sequence(decoder, packet.data + packet.offset, packet.size);
