@@ -465,6 +465,41 @@ gl_swap_buffers (GLContextState * cs)
   cs->swapped_buffers = TRUE;
 }
 
+static inline gboolean
+_init_texture_state (GLTextureState * ts, GLenum target, GLuint texture,
+    gboolean enabled)
+{
+  GLenum binding;
+
+  ts->target = target;
+
+  if (enabled) {
+    binding = gl_get_texture_binding (target);
+    if (!binding)
+      return FALSE;
+    if (!gl_get_param (binding, &ts->old_texture))
+      return FALSE;
+    ts->was_enabled = TRUE;
+    ts->was_bound = texture == ts->old_texture;
+  } else {
+    ts->old_texture = 0;
+    ts->was_enabled = FALSE;
+    ts->was_bound = FALSE;
+  }
+
+  return TRUE;
+}
+
+static inline gboolean
+_bind_enabled_texture (GLenum target, GLuint texture)
+{
+  gl_purge_errors ();
+  glBindTexture (target, texture);
+  if (gl_check_error ())
+    return FALSE;
+  return TRUE;
+}
+
 /**
  * gl_bind_texture:
  * @ts: a #GLTextureState
@@ -479,32 +514,44 @@ gl_swap_buffers (GLContextState * cs)
 gboolean
 gl_bind_texture (GLTextureState * ts, GLenum target, GLuint texture)
 {
-  GLenum binding;
+  gboolean enabled;
 
-  ts->target = target;
-
-  if (glIsEnabled (target)) {
-    binding = gl_get_texture_binding (target);
-    if (!binding)
-      return FALSE;
-    if (!gl_get_param (binding, &ts->old_texture))
-      return FALSE;
-    ts->was_enabled = TRUE;
-    ts->was_bound = texture == ts->old_texture;
-    if (ts->was_bound)
-      return TRUE;
-  } else {
-    glEnable (target);
-    ts->old_texture = 0;
-    ts->was_enabled = FALSE;
-    ts->was_bound = FALSE;
-  }
-
-  gl_purge_errors ();
-  glBindTexture (target, texture);
-  if (gl_check_error ())
+  enabled = (gboolean) glIsEnabled (target);
+  if (!_init_texture_state (ts, target, texture, enabled))
     return FALSE;
-  return TRUE;
+  if (ts->was_bound)
+    return TRUE;
+  if (!enabled)
+    glEnable (target);
+
+  return _bind_enabled_texture (target, texture);
+}
+
+/**
+ * gl3_bind_texture_2d:
+ * @ts: a #GLTextureState
+ * @target: the target to which the texture is bound
+ * @texture: the name of a texture
+ *
+ * Binds @texture to the specified @target, while recording the
+ * previous state in @ts.
+ *
+ * This function is for OpenGL3 API and for targets type GL_TEXTURE_2D.
+ *
+ * Return value: %TRUE on success
+ */
+gboolean
+gl3_bind_texture_2d (GLTextureState * ts, GLenum target, GLuint texture)
+{
+  if (target != GL_TEXTURE_2D)
+    return FALSE;
+
+  if (!_init_texture_state (ts, target, texture, TRUE))
+    return FALSE;
+  if (ts->was_bound)
+    return TRUE;
+
+  return _bind_enabled_texture (target, texture);
 }
 
 /**
@@ -1087,3 +1134,92 @@ gl_unbind_framebuffer_object (GLFramebufferObject * fbo)
   fbo->is_bound = FALSE;
   return TRUE;
 }
+
+/**
+ * gl_get_current_api:
+ * @major: (out): (allow-none): the GL major version
+ * @minor: (out): (allow-none): the GL minor version
+ *
+ * If an error occurs, @major and @minor aren't modified and
+ * %GST_VAAPI_GL_API_NONE is returned.
+ *
+ * This is an adaptation of gst_gl_context_get_current_gl_api() from GstGL.
+ *
+ * Returns: The version supported by the OpenGL context current in the calling
+ *          thread or %GST_VAAPI_GL_API_NONE
+ */
+GstVaapiGLApi
+gl_get_current_api (guint * major, guint * minor)
+{
+  const gchar *version;
+  gint maj, min, n;
+  GstVaapiGLApi ret = (1 << 31);
+
+  while (ret != GST_VAAPI_GL_API_NONE) {
+    version = (const gchar *) glGetString (GL_VERSION);
+    if (!version)
+      goto next;
+
+    /* strlen (x.x) == 3 */
+    n = strlen (version);
+    if (n < 3)
+      goto next;
+
+    if (g_strstr_len (version, 9, "OpenGL ES")) {
+      /* strlen (OpenGL ES x.x) == 13 */
+      if (n < 13)
+        goto next;
+
+      sscanf (&version[10], "%d.%d", &maj, &min);
+
+      if (maj <= 0 || min < 0)
+        goto next;
+
+      if (maj == 1) {
+        ret = GST_VAAPI_GL_API_GLES1;
+        break;
+      } else if (maj == 2 || maj == 3) {
+        ret = GST_VAAPI_GL_API_GLES2;
+        break;
+      }
+
+      goto next;
+    } else {
+      sscanf (version, "%d.%d", &maj, &min);
+
+      if (maj <= 0 || min < 0)
+        goto next;
+
+      if (maj > 3 || (maj == 3 && min > 1)) {
+        GLuint context_flags = 0;
+
+        ret = GST_VAAPI_GL_API_NONE;
+        if (!gl_get_param (GL_CONTEXT_PROFILE_MASK, &context_flags))
+          break;
+
+        if (context_flags & GL_CONTEXT_CORE_PROFILE_BIT)
+          ret |= GST_VAAPI_GL_API_OPENGL3;
+        if (context_flags & GL_CONTEXT_COMPATIBILITY_PROFILE_BIT)
+          ret |= GST_VAAPI_GL_API_OPENGL;
+        break;
+      }
+
+      ret = GST_VAAPI_GL_API_OPENGL;
+      break;
+    }
+
+  next:
+    /* iterate through the apis */
+    ret >>= 1;
+  }
+
+  if (ret == GST_VAAPI_GL_API_NONE)
+    return GST_VAAPI_GL_API_NONE;
+
+  if (major)
+    *major = maj;
+  if (minor)
+    *minor = min;
+
+  return ret;
+}
