@@ -109,7 +109,7 @@ struct _GstVaapiWindowWaylandPrivate
   guint is_shown:1;
   guint fullscreen_on_show:1;
   guint use_vpp:1;
-  guint is_cancelled:1;
+  guint sync_failed:1;
   volatile guint num_frames_pending;
 };
 
@@ -161,7 +161,7 @@ gst_vaapi_window_wayland_sync (GstVaapiWindow * window)
   struct wl_display *const wl_display =
       GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
 
-  if (priv->is_cancelled)
+  if (priv->sync_failed)
     return FALSE;
 
   if (priv->pollfd.fd < 0) {
@@ -186,8 +186,7 @@ gst_vaapi_window_wayland_sync (GstVaapiWindow * window)
         goto again;
       if (saved_errno == EBUSY) {       /* closing */
         wl_display_cancel_read (wl_display);
-        priv->is_cancelled = TRUE;
-        break;
+        return FALSE;
       }
       goto error;
     }
@@ -200,7 +199,7 @@ gst_vaapi_window_wayland_sync (GstVaapiWindow * window)
   return TRUE;
 
 error:
-  priv->is_cancelled = TRUE;
+  priv->sync_failed = TRUE;
   GST_ERROR ("Error on dispatching events: %s", g_strerror (errno));
   return FALSE;
 }
@@ -518,12 +517,7 @@ gst_vaapi_window_wayland_render (GstVaapiWindow * window,
   /* Wait for the previous frame to complete redraw */
   if (!gst_vaapi_window_wayland_sync (window)) {
     wl_buffer_destroy (buffer);
-    return FALSE;
-  }
-
-  if (priv->is_cancelled) {
-    wl_buffer_destroy (buffer);
-    return TRUE;
+    return !priv->sync_failed;
   }
 
   frame = frame_state_new (window);
