@@ -49,6 +49,7 @@
 #include <gst/vaapi/gstvaapidecoder_vc1.h>
 #include <gst/vaapi/gstvaapidecoder_vp8.h>
 #include <gst/vaapi/gstvaapidecoder_h265.h>
+#include <gst/vaapi/gstvaapidecoder_vp9.h>
 
 #define GST_PLUGIN_NAME "vaapidecode"
 #define GST_PLUGIN_DESC "A VA-API based video decoder"
@@ -78,14 +79,15 @@ static const char gst_vaapidecode_sink_caps_str[] =
 #endif
 #if USE_JPEG_DECODER
     GST_CAPS_CODEC("image/jpeg")
+#endif
+#if USE_VP9_DECODER
+    GST_CAPS_CODEC("video/x-vp9")
 #endif
     ;
 
 static const char gst_vaapidecode_src_caps_str[] =
-    GST_VIDEO_CAPS_MAKE_WITH_FEATURES(
-        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE, "{ ENCODED, I420, YV12, NV12 }") ";"
-    GST_VIDEO_CAPS_MAKE_WITH_FEATURES(
-        GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "{ RGBA, BGRA }") ";"
+    GST_VAAPI_MAKE_SURFACE_CAPS ";"
+    GST_VAAPI_MAKE_GLTEXUPLOAD_CAPS ";"
     GST_VIDEO_CAPS_MAKE("{ I420, YV12, NV12 }");
 
 static GstStaticPadTemplate gst_vaapidecode_sink_factory =
@@ -109,6 +111,8 @@ G_DEFINE_TYPE_WITH_CODE(
     GST_VAAPI_PLUGIN_BASE_INIT_INTERFACES)
 /* *INDENT-ON* */
 
+static gboolean
+gst_vaapidecode_update_sink_caps (GstVaapiDecode * decode, GstCaps * caps);
 static gboolean gst_vaapidecode_update_src_caps (GstVaapiDecode * decode);
 
 static gboolean
@@ -125,19 +129,15 @@ gst_vaapi_decoder_state_changed (GstVaapiDecoder * decoder,
     const GstVideoCodecState * codec_state, gpointer user_data)
 {
   GstVaapiDecode *const decode = GST_VAAPIDECODE (user_data);
-  GstVideoDecoder *const vdec = GST_VIDEO_DECODER (decode);
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (vdec);
 
   g_assert (decode->decoder == decoder);
 
   if (!gst_vaapi_decode_input_state_replace (decode, codec_state))
     return;
-  if (!gst_vaapidecode_update_src_caps (decode))
-    return;
-  if (!gst_video_decoder_negotiate (vdec))
-    return;
-  if (!gst_vaapi_plugin_base_set_caps (plugin, NULL, decode->srcpad_caps))
+  if (!gst_vaapidecode_update_sink_caps (decode, decode->input_state->caps))
     return;
+
+  decode->do_renego = TRUE;
 }
 
 static GstVideoCodecState *
@@ -151,7 +151,8 @@ copy_video_codec_state (const GstVideoCodecState * in_state)
   state->ref_count = 1;
   state->info = in_state->info;
   state->caps = gst_caps_copy (in_state->caps);
-  state->codec_data = gst_buffer_copy_deep (in_state->codec_data);
+  if (in_state->codec_data)
+    state->codec_data = gst_buffer_copy_deep (in_state->codec_data);
 
   return state;
 }
@@ -216,15 +217,12 @@ gst_vaapidecode_update_src_caps (GstVaapiDecode * decode)
   switch (feature) {
 #if (USE_GLX || USE_EGL)
     case GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META:
-      if (decode->has_texture_upload_meta)
-        features =
-            gst_caps_features_new
-            (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, NULL);
-      else
-        format = GST_VIDEO_FORMAT_I420;
+      features =
+          gst_caps_features_new
+          (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, NULL);
       break;
 #endif
-#if GST_CHECK_VERSION(1,5,0)
+#if GST_CHECK_VERSION(1,3,1)
     case GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE:
       features =
           gst_caps_features_new (GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE, NULL);
@@ -284,7 +282,7 @@ gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
   GstFlowReturn ret;
   const GstVaapiRectangle *crop_rect;
   GstVaapiVideoMeta *meta;
-  guint flags;
+  guint flags, out_flags = 0;
 
   if (!GST_VIDEO_CODEC_FRAME_IS_DECODE_ONLY (out_frame)) {
     proxy = gst_video_codec_frame_get_user_data (out_frame);
@@ -302,16 +300,26 @@ gst_vaapidecode_push_decoded_frame (GstVideoDecoder * vdec,
     gst_vaapi_video_meta_set_surface_proxy (meta, proxy);
 
     flags = gst_vaapi_surface_proxy_get_flags (proxy);
+    if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_CORRUPTED)
+      out_flags |= GST_BUFFER_FLAG_CORRUPTED;
     if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_INTERLACED) {
-      guint out_flags = GST_VIDEO_BUFFER_FLAG_INTERLACED;
+      out_flags |= GST_VIDEO_BUFFER_FLAG_INTERLACED;
       if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_TFF)
         out_flags |= GST_VIDEO_BUFFER_FLAG_TFF;
       if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_RFF)
         out_flags |= GST_VIDEO_BUFFER_FLAG_RFF;
       if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_ONEFIELD)
         out_flags |= GST_VIDEO_BUFFER_FLAG_ONEFIELD;
-      GST_BUFFER_FLAG_SET (out_frame->output_buffer, out_flags);
     }
+    GST_BUFFER_FLAG_SET (out_frame->output_buffer, out_flags);
+
+#if GST_CHECK_VERSION(1,5,0)
+    /* First-in-bundle flag only appeared in 1.5 dev */
+    if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_FFB) {
+      GST_BUFFER_FLAG_SET (out_frame->output_buffer,
+          GST_VIDEO_BUFFER_FLAG_FIRST_IN_BUNDLE);
+    }
+#endif
 
     crop_rect = gst_vaapi_surface_proxy_get_crop_rect (proxy);
     if (crop_rect) {
@@ -369,6 +377,31 @@ error_commit_buffer:
   }
 }
 
+static gboolean
+gst_vaapidecode_negotiate (GstVaapiDecode * decode)
+{
+  GstVideoDecoder *const vdec = GST_VIDEO_DECODER (decode);
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (vdec);
+
+  if (!decode->do_renego)
+    return TRUE;
+
+  GST_DEBUG_OBJECT (decode, "Input codec state changed, doing renegotiation");
+
+  if (!gst_vaapi_plugin_base_set_caps (plugin, decode->sinkpad_caps, NULL))
+    return FALSE;
+  if (!gst_vaapidecode_update_src_caps (decode))
+    return FALSE;
+  if (!gst_video_decoder_negotiate (vdec))
+    return FALSE;
+  if (!gst_vaapi_plugin_base_set_caps (plugin, NULL, decode->srcpad_caps))
+    return FALSE;
+
+  decode->do_renego = FALSE;
+
+  return TRUE;
+}
+
 static GstFlowReturn
 gst_vaapidecode_push_all_decoded_frames (GstVaapiDecode * decode)
 {
@@ -387,6 +420,12 @@ gst_vaapidecode_push_all_decoded_frames (GstVaapiDecode * decode)
           return ret;
         break;
       case GST_VAAPI_DECODER_STATUS_ERROR_NO_DATA:
+        /* Delayed the pool re-negotiation untill we push all decoded (and queued)
+         * frames downstream. Otherwise for the multi-resolution videos, the
+         * GstVideoVideoMemory will be having wrong resolution.
+         * commit 6eba201f3252eba6a99ab7da7a4c662091a3e884 */
+        if (!gst_vaapidecode_negotiate (decode))
+          return GST_FLOW_ERROR;
         return GST_FLOW_OK;
       default:
         GST_VIDEO_DECODER_ERROR (vdec, 1, STREAM, DECODE, ("Decoding failed"),
@@ -544,31 +583,19 @@ gst_vaapidecode_decide_allocation (GstVideoDecoder * vdec, GstQuery * query)
 {
   GstVaapiDecode *const decode = GST_VAAPIDECODE (vdec);
   GstCaps *caps = NULL;
-  GstVideoCodecState *state;
-  GstVaapiCapsFeature feature;
-  GstVideoFormat out_format;
 
   gst_query_parse_allocation (query, &caps, NULL);
-
-  feature =
-      gst_vaapi_find_preferred_caps_feature (GST_VIDEO_DECODER_SRC_PAD (vdec),
-      GST_VIDEO_FORMAT_ENCODED, &out_format);
   decode->has_texture_upload_meta = FALSE;
 #if (USE_GLX || USE_EGL)
   decode->has_texture_upload_meta =
-      (feature == GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META) &&
       gst_query_find_allocation_meta (query,
-      GST_VIDEO_GL_TEXTURE_UPLOAD_META_API_TYPE, NULL);
+      GST_VIDEO_GL_TEXTURE_UPLOAD_META_API_TYPE, NULL) &&
+      gst_vaapi_caps_feature_contains (caps,
+      GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META);
 #endif
 
-  /* Update src caps if feature is not handled downstream */
-  state = gst_video_decoder_get_output_state (vdec);
-  if (!gst_caps_is_always_compatible (caps, state->caps))
-    gst_vaapidecode_update_src_caps (decode);
-  gst_video_codec_state_unref (state);
-
   return gst_vaapi_plugin_base_decide_allocation (GST_VAAPI_PLUGIN_BASE (vdec),
-      query, feature);
+      query, 0);
 }
 
 static inline gboolean
@@ -657,6 +684,11 @@ gst_vaapidecode_create (GstVaapiDecode * decode, GstCaps * caps)
     case GST_VAAPI_CODEC_VP8:
       decode->decoder = gst_vaapi_decoder_vp8_new (dpy, caps);
       break;
+#endif
+#if USE_VP9_DECODER
+    case GST_VAAPI_CODEC_VP9:
+      decode->decoder = gst_vaapi_decoder_vp9_new (dpy, caps);
+      break;
 #endif
     default:
       decode->decoder = NULL;
@@ -926,12 +958,6 @@ gst_vaapidecode_ensure_allowed_caps (GstVaapiDecode * decode)
   GArray *profiles;
   guint i;
 
-  if (decode->allowed_caps)
-    return TRUE;
-
-  if (!gst_vaapidecode_ensure_display (decode))
-    goto error_no_display;
-
   profiles =
       gst_vaapi_display_get_decode_profiles (GST_VAAPI_PLUGIN_BASE_DISPLAY
       (decode));
@@ -971,11 +997,6 @@ gst_vaapidecode_ensure_allowed_caps (GstVaapiDecode * decode)
   return TRUE;
 
   /* ERRORS */
-error_no_display:
-  {
-    GST_ERROR ("failed to retrieve VA display");
-    return FALSE;
-  }
 error_no_profiles:
   {
     GST_ERROR ("failed to retrieve VA decode profiles");
@@ -994,9 +1015,19 @@ gst_vaapidecode_get_caps (GstPad * pad)
 {
   GstVaapiDecode *const decode = GST_VAAPIDECODE (GST_OBJECT_PARENT (pad));
 
+  if (decode->allowed_caps)
+    goto bail;
+
+  /* if we haven't a display yet, return our pad's template caps */
+  if (!GST_VAAPI_PLUGIN_BASE_DISPLAY (decode))
+    return gst_pad_get_pad_template_caps (pad);
+
+  /* if the allowed caps calculation fails, return an empty caps, so
+   * the auto-plug can try other decoder */
   if (!gst_vaapidecode_ensure_allowed_caps (decode))
     return gst_caps_new_empty ();
 
+bail:
   return gst_caps_ref (decode->allowed_caps);
 }
 
@@ -1029,11 +1060,6 @@ gst_vaapidecode_sink_query (GstVideoDecoder * vdec, GstQuery * query)
   GstVaapiDecode *const decode = GST_VAAPIDECODE (vdec);
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (decode);
 
-  if (gst_vaapi_reply_to_query (query, plugin->display)) {
-    GST_DEBUG_OBJECT (decode, "sharing display %p", plugin->display);
-    return TRUE;
-  }
-
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CAPS:{
       GstCaps *caps, *filter = NULL;
@@ -1052,6 +1078,10 @@ gst_vaapidecode_sink_query (GstVideoDecoder * vdec, GstQuery * query)
       gst_caps_unref (caps);
       break;
     }
+    case GST_QUERY_CONTEXT:{
+      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      break;
+    }
     default:{
 #if GST_CHECK_VERSION(1,4,0)
       ret = GST_VIDEO_DECODER_CLASS (gst_vaapidecode_parent_class)->sink_query
@@ -1077,11 +1107,6 @@ gst_vaapidecode_src_query (GstVideoDecoder * vdec, GstQuery * query)
   GstVaapiDecode *const decode = GST_VAAPIDECODE (vdec);
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (decode);
 
-  if (gst_vaapi_reply_to_query (query, plugin->display)) {
-    GST_DEBUG_OBJECT (decode, "sharing display %p", plugin->display);
-    return TRUE;
-  }
-
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CAPS:{
       GstCaps *caps, *filter = NULL;
@@ -1100,6 +1125,10 @@ gst_vaapidecode_src_query (GstVideoDecoder * vdec, GstQuery * query)
       gst_caps_unref (caps);
       break;
     }
+    case GST_QUERY_CONTEXT:{
+      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      break;
+    }
     default:{
 #if GST_CHECK_VERSION(1,4,0)
       ret = GST_VIDEO_DECODER_CLASS (gst_vaapidecode_parent_class)->src_query
