@@ -57,16 +57,45 @@ ensure_display (GstVaapiEncode * encode)
   return gst_vaapi_plugin_base_ensure_display (GST_VAAPI_PLUGIN_BASE (encode));
 }
 
+#if GST_CHECK_VERSION(1,4,0)
 static gboolean
-ensure_uploader (GstVaapiEncode * encode)
+gst_vaapiencode_sink_query (GstVideoEncoder * encoder, GstQuery * query)
 {
-  if (!ensure_display (encode))
-    return FALSE;
-  if (!gst_vaapi_plugin_base_ensure_uploader (GST_VAAPI_PLUGIN_BASE (encode)))
-    return FALSE;
-  return TRUE;
+  gboolean ret = TRUE;
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (encoder);
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CONTEXT:
+      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      break;
+    default:
+      ret = GST_VIDEO_ENCODER_CLASS (gst_vaapiencode_parent_class)->sink_query
+          (encoder, query);
+      break;
+  }
+
+  return ret;
 }
 
+static gboolean
+gst_vaapiencode_src_query (GstVideoEncoder * encoder, GstQuery * query)
+{
+  gboolean ret = TRUE;
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (encoder);
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CONTEXT:
+      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      break;
+    default:
+      ret = GST_VIDEO_ENCODER_CLASS (gst_vaapiencode_parent_class)->src_query
+          (encoder, query);
+      break;
+  }
+
+  return ret;
+}
+#else
 static gboolean
 gst_vaapiencode_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
@@ -76,8 +105,8 @@ gst_vaapiencode_query (GstPad * pad, GstObject * parent, GstQuery * query)
 
   GST_INFO_OBJECT (plugin, "query type %s", GST_QUERY_TYPE_NAME (query));
 
-  if (gst_vaapi_reply_to_query (query, plugin->display))
-    success = TRUE;
+  if (GST_QUERY_TYPE (query) == GST_QUERY_CONTEXT)
+    success = gst_vaapi_handle_context_query (query, plugin->display);
   else if (GST_PAD_IS_SINK (pad))
     success = plugin->sinkpad_query (plugin->sinkpad, parent, query);
   else
@@ -86,6 +115,7 @@ gst_vaapiencode_query (GstPad * pad, GstObject * parent, GstQuery * query)
   gst_object_unref (plugin);
   return success;
 }
+#endif
 
 typedef struct
 {
@@ -385,9 +415,6 @@ ensure_encoder (GstVaapiEncode * encode)
 
   g_return_val_if_fail (klass->alloc_encoder, FALSE);
 
-  if (!ensure_uploader (encode))
-    return FALSE;
-
   encode->encoder = klass->alloc_encoder (encode,
       GST_VAAPI_PLUGIN_BASE_DISPLAY (encode));
   if (!encode->encoder)
@@ -623,8 +650,10 @@ gst_vaapiencode_init (GstVaapiEncode * encode)
 
   gst_vaapi_plugin_base_init (GST_VAAPI_PLUGIN_BASE (encode), GST_CAT_DEFAULT);
 
+#if !GST_CHECK_VERSION(1,4,0)
   gst_pad_set_query_function (plugin->sinkpad, gst_vaapiencode_query);
   gst_pad_set_query_function (plugin->srcpad, gst_vaapiencode_query);
+#endif
   gst_pad_use_fixed_caps (plugin->srcpad);
 }
 
@@ -658,8 +687,13 @@ gst_vaapiencode_class_init (GstVaapiEncodeClass * klass)
   klass->set_property = gst_vaapiencode_default_set_property;
   klass->alloc_buffer = gst_vaapiencode_default_alloc_buffer;
 
+#if GST_CHECK_VERSION(1,4,0)
+  venc_class->src_query = GST_DEBUG_FUNCPTR (gst_vaapiencode_src_query);
+  venc_class->sink_query = GST_DEBUG_FUNCPTR (gst_vaapiencode_sink_query);
+#else
   /* Registering debug symbols for function pointers */
   GST_DEBUG_REGISTER_FUNCPTR (gst_vaapiencode_query);
+#endif
 }
 
 static inline GPtrArray *
