@@ -35,6 +35,8 @@
 /* Default debug category is from the subclass */
 #define GST_CAT_DEFAULT (plugin->debug_category)
 
+static gpointer plugin_parent_class = NULL;
+
 /* GstVideoContext interface */
 static void
 plugin_set_display (GstVaapiPluginBase * plugin, GstVaapiDisplay * display)
@@ -59,10 +61,14 @@ static void
 plugin_set_context (GstElement * element, GstContext * context)
 {
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (element);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (plugin_parent_class);
   GstVaapiDisplay *display = NULL;
 
   if (gst_vaapi_video_context_get_display (context, &display))
     plugin_set_display (plugin, display);
+
+  if (element_class->set_context)
+    element_class->set_context (element, context);
 }
 
 void
@@ -177,6 +183,8 @@ gst_vaapi_plugin_base_class_init (GstVaapiPluginBaseClass * klass)
   klass->has_interface = default_has_interface;
   klass->display_changed = default_display_changed;
 
+  plugin_parent_class = g_type_class_peek_parent (klass);
+
   GstElementClass *const element_class = GST_ELEMENT_CLASS (klass);
   element_class->set_context = GST_DEBUG_FUNCPTR (plugin_set_context);
 }
@@ -191,13 +199,17 @@ gst_vaapi_plugin_base_init (GstVaapiPluginBase * plugin,
 
   /* sink pad */
   plugin->sinkpad = gst_element_get_static_pad (GST_ELEMENT (plugin), "sink");
-  plugin->sinkpad_query = GST_PAD_QUERYFUNC (plugin->sinkpad);
   gst_video_info_init (&plugin->sinkpad_info);
+#if !GST_CHECK_VERSION(1,4,0)
+  plugin->sinkpad_query = GST_PAD_QUERYFUNC (plugin->sinkpad);
+#endif
 
   /* src pad */
   if (!(GST_OBJECT_FLAGS (plugin) & GST_ELEMENT_FLAG_SINK)) {
     plugin->srcpad = gst_element_get_static_pad (GST_ELEMENT (plugin), "src");
+#if !GST_CHECK_VERSION(1,4,0)
     plugin->srcpad_query = GST_PAD_QUERYFUNC (plugin->srcpad);
+#endif
   }
   gst_video_info_init (&plugin->srcpad_info);
 }
@@ -238,7 +250,6 @@ gst_vaapi_plugin_base_open (GstVaapiPluginBase * plugin)
 void
 gst_vaapi_plugin_base_close (GstVaapiPluginBase * plugin)
 {
-  g_clear_object (&plugin->uploader);
   gst_vaapi_display_replace (&plugin->display, NULL);
   gst_object_replace (&plugin->gl_context, NULL);
 
@@ -254,6 +265,7 @@ gst_vaapi_plugin_base_close (GstVaapiPluginBase * plugin)
   gst_caps_replace (&plugin->srcpad_caps, NULL);
   plugin->srcpad_caps_changed = FALSE;
   gst_video_info_init (&plugin->srcpad_info);
+  gst_caps_replace (&plugin->allowed_raw_caps, NULL);
 }
 
 /**
@@ -342,29 +354,6 @@ gst_vaapi_plugin_base_ensure_display (GstVaapiPluginBase * plugin)
   return TRUE;
 }
 
-/**
- * gst_vaapi_plugin_base_ensure_uploader:
- * @plugin: a #GstVaapiPluginBase
- *
- * Makes sure the built-in #GstVaapiUploader object is created, or
- * that it was successfully notified of any VA display change.
- *
- * Returns: %TRUE if the uploader was successfully created, %FALSE otherwise.
- */
-gboolean
-gst_vaapi_plugin_base_ensure_uploader (GstVaapiPluginBase * plugin)
-{
-  if (plugin->uploader) {
-    if (!gst_vaapi_uploader_ensure_display (plugin->uploader, plugin->display))
-      return FALSE;
-  } else {
-    plugin->uploader = gst_vaapi_uploader_new (plugin->display);
-    if (!plugin->uploader)
-      return FALSE;
-  }
-  return TRUE;
-}
-
 /* Checks whether the supplied pad peer element supports DMABUF sharing */
 /* XXX: this is a workaround to the absence of any proposer way to
    specify DMABUF memory capsfeatures or bufferpool option to downstream */
@@ -521,14 +510,6 @@ gst_vaapi_plugin_base_set_caps (GstVaapiPluginBase * plugin, GstCaps * incaps,
     plugin->srcpad_caps_changed = TRUE;
   }
 
-  if (plugin->uploader && plugin->sinkpad_caps_is_raw) {
-    if (!gst_vaapi_uploader_ensure_display (plugin->uploader, plugin->display))
-      return FALSE;
-    if (!gst_vaapi_uploader_ensure_caps (plugin->uploader,
-            plugin->sinkpad_caps, plugin->srcpad_caps))
-      return FALSE;
-  }
-
   if (!ensure_sinkpad_buffer_pool (plugin, plugin->sinkpad_caps))
     return FALSE;
   return TRUE;
@@ -578,12 +559,12 @@ gst_vaapi_plugin_base_propose_allocation (GstVaapiPluginBase * plugin,
   /* ERRORS */
 error_no_caps:
   {
-    GST_ERROR ("no caps specified");
+    GST_INFO_OBJECT (plugin, "no caps specified");
     return FALSE;
   }
 error_pool_config:
   {
-    GST_ERROR ("failed to reset buffer pool config");
+    GST_ERROR_OBJECT (plugin, "failed to reset buffer pool config");
     return FALSE;
   }
 }
@@ -777,49 +758,6 @@ config_failed:
   }
 }
 
-/**
- * gst_vaapi_plugin_base_allocate_input_buffer:
- * @plugin: a #GstVaapiPluginBase
- * @caps: the buffer caps constraints to honour
- * @outbuf_ptr: the pointer location to the newly allocated buffer
- *
- * Creates a buffer that holds a VA surface memory for the sink pad to
- * use it as the result for buffer_alloc() impementations.
- *
- * Return: #GST_FLOW_OK if the buffer could be created.
- */
-GstFlowReturn
-gst_vaapi_plugin_base_allocate_input_buffer (GstVaapiPluginBase * plugin,
-    GstCaps * caps, GstBuffer ** outbuf_ptr)
-{
-  GstBuffer *outbuf;
-
-  *outbuf_ptr = NULL;
-
-  if (!plugin->sinkpad_caps_changed) {
-    if (!gst_video_info_from_caps (&plugin->sinkpad_info, caps))
-      return GST_FLOW_NOT_SUPPORTED;
-    plugin->sinkpad_caps_changed = TRUE;
-  }
-
-  if (!plugin->sinkpad_caps_is_raw)
-    return GST_FLOW_OK;
-
-  if (!gst_vaapi_uploader_ensure_display (plugin->uploader, plugin->display))
-    return GST_FLOW_NOT_SUPPORTED;
-  if (!gst_vaapi_uploader_ensure_caps (plugin->uploader, caps, NULL))
-    return GST_FLOW_NOT_SUPPORTED;
-
-  outbuf = gst_vaapi_uploader_get_buffer (plugin->uploader);
-  if (!outbuf) {
-    GST_WARNING ("failed to allocate resources for raw YUV buffer");
-    return GST_FLOW_NOT_SUPPORTED;
-  }
-
-  *outbuf_ptr = outbuf;
-  return GST_FLOW_OK;
-}
-
 /**
  * gst_vaapi_plugin_base_get_input_buffer:
  * @plugin: a #GstVaapiPluginBase
@@ -983,3 +921,82 @@ gst_vaapi_plugin_base_set_gl_context (GstVaapiPluginBase * plugin,
   gst_vaapi_plugin_base_set_display_type (plugin, display_type);
 #endif
 }
+
+static gboolean
+ensure_allowed_raw_caps (GstVaapiPluginBase * plugin)
+{
+  GArray *formats, *out_formats;
+  GstVaapiSurface *surface;
+  guint i;
+  GstCaps *out_caps;
+  gboolean ret = FALSE;
+
+  if (plugin->allowed_raw_caps)
+    return TRUE;
+
+  out_formats = formats = NULL;
+  surface = NULL;
+
+  formats = gst_vaapi_display_get_image_formats (plugin->display);
+  if (!formats)
+    goto bail;
+
+  out_formats =
+      g_array_sized_new (FALSE, FALSE, sizeof (GstVideoFormat), formats->len);
+  if (!out_formats)
+    goto bail;
+
+  surface =
+      gst_vaapi_surface_new (plugin->display, GST_VAAPI_CHROMA_TYPE_YUV420, 64,
+      64);
+  if (!surface)
+    goto bail;
+
+  for (i = 0; i < formats->len; i++) {
+    const GstVideoFormat format = g_array_index (formats, GstVideoFormat, i);
+    GstVaapiImage *image;
+
+    if (format == GST_VIDEO_FORMAT_UNKNOWN)
+      continue;
+    image = gst_vaapi_image_new (plugin->display, format, 64, 64);
+    if (!image)
+      continue;
+    if (gst_vaapi_surface_put_image (surface, image))
+      g_array_append_val (out_formats, format);
+    gst_vaapi_object_unref (image);
+  }
+
+  out_caps = gst_vaapi_video_format_new_template_caps_from_list (out_formats);
+  if (!out_caps)
+    goto bail;
+
+  gst_caps_replace (&plugin->allowed_raw_caps, out_caps);
+  gst_caps_unref (out_caps);
+  ret = TRUE;
+
+bail:
+  if (formats)
+    g_array_unref (formats);
+  if (out_formats)
+    g_array_unref (out_formats);
+  if (surface)
+    gst_vaapi_object_unref (surface);
+
+  return ret;
+}
+
+/**
+ * gst_vaapi_plugin_base_get_allowed_raw_caps:
+ * @plugin: a #GstVaapiPluginBase
+ *
+ * Returns the raw #GstCaps allowed by the element.
+ *
+ * Returns: the allowed raw #GstCaps or %NULL
+ **/
+GstCaps *
+gst_vaapi_plugin_base_get_allowed_raw_caps (GstVaapiPluginBase * plugin)
+{
+  if (!ensure_allowed_raw_caps (plugin))
+    return NULL;
+  return plugin->allowed_raw_caps;
+}
