@@ -42,25 +42,6 @@
 #include "gstvaapipluginutil.h"
 #include "gstvaapipluginbase.h"
 
-/* Preferred first */
-static const char *display_types[] = {
-  "gst-vaapi-display",
-  "vaapi-display",
-#if USE_WAYLAND
-  "wl-display",
-  "wl-display-name",
-#endif
-#if USE_X11
-  "x11-display",
-  "x11-display-name",
-#endif
-#if USE_DRM
-  "drm-device",
-  "drm-device-path",
-#endif
-  NULL
-};
-
 typedef GstVaapiDisplay *(*GstVaapiDisplayCreateFunc) (const gchar *);
 typedef GstVaapiDisplay *(*GstVaapiDisplayCreateFromHandleFunc) (gpointer);
 
@@ -254,11 +235,11 @@ gst_vaapi_ensure_display (GstElement * element, GstVaapiDisplayType type)
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  gst_vaapi_video_context_prepare (element, display_types);
-
-  /* Neighbour found and it updated the display */
-  if (gst_vaapi_plugin_base_has_display_type (plugin, type))
-    return TRUE;
+  if (gst_vaapi_video_context_prepare (element, &plugin->display)) {
+    /* Neighbour found and it updated the display */
+    if (gst_vaapi_plugin_base_has_display_type (plugin, type))
+      return TRUE;
+  }
 
   /* If no neighboor, or application not interested, use system default */
   if (plugin->gl_context)
@@ -269,19 +250,17 @@ gst_vaapi_ensure_display (GstElement * element, GstVaapiDisplayType type)
     return FALSE;
 
   gst_vaapi_video_context_propagate (element, display);
-  GST_VAAPI_PLUGIN_BASE_DISPLAY_REPLACE (plugin, display);
   gst_vaapi_display_unref (display);
   return TRUE;
 }
 
 gboolean
-gst_vaapi_reply_to_query (GstQuery * query, GstVaapiDisplay * display)
+gst_vaapi_handle_context_query (GstQuery * query, GstVaapiDisplay * display)
 {
   const gchar *type = NULL;
-  GstContext *context;
+  GstContext *context, *old_context;
 
-  if (GST_QUERY_TYPE (query) != GST_QUERY_CONTEXT)
-    return FALSE;
+  g_return_val_if_fail (query != NULL, FALSE);
 
   if (!display)
     return FALSE;
@@ -292,7 +271,14 @@ gst_vaapi_reply_to_query (GstQuery * query, GstVaapiDisplay * display)
   if (g_strcmp0 (type, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME))
     return FALSE;
 
-  context = gst_vaapi_video_context_new_with_display (display, FALSE);
+  gst_query_parse_context (query, &old_context);
+  if (old_context) {
+    context = gst_context_copy (old_context);
+    gst_vaapi_video_context_set_display (context, display);
+  } else {
+    context = gst_vaapi_video_context_new_with_display (display, FALSE);
+  }
+
   gst_query_set_context (query, context);
   gst_context_unref (context);
 
@@ -609,32 +595,44 @@ gst_caps_set_interlaced (GstCaps * caps, GstVideoInfo * vip)
   return TRUE;
 }
 
+static gboolean
+_gst_caps_has_feature (const GstCaps * caps, const gchar * feature)
+{
+  guint i;
+
+  for (i = 0; i < gst_caps_get_size (caps); i++) {
+    GstCapsFeatures *const features = gst_caps_get_features (caps, i);
+    /* Skip ANY features, we need an exact match for correct evaluation */
+    if (gst_caps_features_is_any (features))
+      continue;
+    if (gst_caps_features_contains (features, feature))
+      return TRUE;
+  }
+
+  return FALSE;
+}
+
+gboolean
+gst_vaapi_caps_feature_contains (const GstCaps * caps, GstVaapiCapsFeature feature)
+{
+  const gchar *feature_str;
+
+  g_return_val_if_fail (caps != NULL, FALSE);
+
+  feature_str = gst_vaapi_caps_feature_to_string (feature);
+  if (!feature_str)
+    return FALSE;
+
+  return _gst_caps_has_feature (caps, feature_str);
+}
+
 /* Checks whether the supplied caps contain VA surfaces */
 gboolean
 gst_caps_has_vaapi_surface (GstCaps * caps)
 {
-  gboolean found_caps = FALSE;
-  guint i, num_structures;
-
   g_return_val_if_fail (caps != NULL, FALSE);
 
-  num_structures = gst_caps_get_size (caps);
-  if (num_structures < 1)
-    return FALSE;
-
-  for (i = 0; i < num_structures && !found_caps; i++) {
-    GstCapsFeatures *const features = gst_caps_get_features (caps, i);
-
-#if GST_CHECK_VERSION(1,3,0)
-    /* Skip ANY features, we need an exact match for correct evaluation */
-    if (gst_caps_features_is_any (features))
-      continue;
-#endif
-
-    found_caps = gst_caps_features_contains (features,
-        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
-  }
-  return found_caps;
+  return _gst_caps_has_feature (caps, GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
 }
 
 void
@@ -652,4 +650,26 @@ gst_video_info_change_format (GstVideoInfo * vip, GstVideoFormat format,
   vip->par_d = vi.par_d;
   vip->fps_n = vi.fps_n;
   vip->fps_d = vi.fps_d;
+
+#if GST_CHECK_VERSION(1,5,0)
+  GST_VIDEO_INFO_MULTIVIEW_MODE (vip) =
+      GST_VIDEO_INFO_MULTIVIEW_MODE (&vi);
+  GST_VIDEO_INFO_MULTIVIEW_FLAGS (vip) =
+      GST_VIDEO_INFO_MULTIVIEW_FLAGS (&vi);
+#endif
+}
+
+/**
+ * gst_vaapi_create_test_display:
+ *
+ * Creates a temporal #GstVaapiDisplay instance, just for testing the
+ * supported features.
+ *
+ * Returns: a new #GstVaapiDisplay instances. Free with
+ * gst_vaapi_display_unref () after use.
+ **/
+GstVaapiDisplay *
+gst_vaapi_create_test_display ()
+{
+  return gst_vaapi_create_display (GST_VAAPI_DISPLAY_TYPE_ANY, NULL);
 }
