@@ -44,26 +44,21 @@
 GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapipostproc);
 #define GST_CAT_DEFAULT gst_debug_vaapipostproc
 
-# define GST_VAAPIPOSTPROC_SURFACE_CAPS \
-    GST_VIDEO_CAPS_MAKE_WITH_FEATURES(  \
-        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE, "{ ENCODED, I420, YV12, NV12 }")
-
 /* Default templates */
 /* *INDENT-OFF* */
 static const char gst_vaapipostproc_sink_caps_str[] =
-  GST_VAAPIPOSTPROC_SURFACE_CAPS ", "
+  GST_VAAPI_MAKE_SURFACE_CAPS ", "
   GST_CAPS_INTERLACED_MODES "; "
+  GST_VAAPI_MAKE_GLTEXUPLOAD_CAPS "; "
   GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
    GST_CAPS_INTERLACED_MODES;
 /* *INDENT-ON* */
 
 /* *INDENT-OFF* */
 static const char gst_vaapipostproc_src_caps_str[] =
-  GST_VAAPIPOSTPROC_SURFACE_CAPS ", "
-  GST_CAPS_INTERLACED_FALSE "; "
-  GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
-      GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, "{ RGBA, BGRA }") ", "
+  GST_VAAPI_MAKE_SURFACE_CAPS ", "
   GST_CAPS_INTERLACED_FALSE "; "
+  GST_VAAPI_MAKE_GLTEXUPLOAD_CAPS "; "
   GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
   GST_CAPS_INTERLACED_FALSE;
 /* *INDENT-ON* */
@@ -213,16 +208,6 @@ gst_vaapipostproc_ensure_display (GstVaapiPostproc * postproc)
       gst_vaapi_plugin_base_ensure_display (GST_VAAPI_PLUGIN_BASE (postproc));
 }
 
-static gboolean
-gst_vaapipostproc_ensure_uploader (GstVaapiPostproc * postproc)
-{
-  if (!gst_vaapipostproc_ensure_display (postproc))
-    return FALSE;
-  if (!gst_vaapi_plugin_base_ensure_uploader (GST_VAAPI_PLUGIN_BASE (postproc)))
-    return FALSE;
-  return TRUE;
-}
-
 static gboolean
 gst_vaapipostproc_ensure_filter (GstVaapiPostproc * postproc)
 {
@@ -265,8 +250,6 @@ gst_vaapipostproc_create (GstVaapiPostproc * postproc)
     return FALSE;
   if (!gst_vaapipostproc_ensure_display (postproc))
     return FALSE;
-  if (!gst_vaapipostproc_ensure_uploader (postproc))
-    return FALSE;
 
   postproc->use_vpp = FALSE;
   postproc->has_vpp = gst_vaapipostproc_ensure_filter (postproc);
@@ -872,6 +855,24 @@ video_info_changed (GstVideoInfo * old_vip, GstVideoInfo * new_vip)
   return FALSE;
 }
 
+static gboolean
+video_info_update (GstCaps * caps, GstVideoInfo * info,
+    gboolean * caps_changed_ptr)
+{
+  GstVideoInfo vi;
+
+  if (!gst_video_info_from_caps (&vi, caps))
+    return FALSE;
+
+  *caps_changed_ptr = FALSE;
+  if (video_info_changed (info, &vi)) {
+    *caps_changed_ptr = TRUE;
+    *info = vi;
+  }
+
+  return TRUE;
+}
+
 static gboolean
 gst_vaapipostproc_update_sink_caps (GstVaapiPostproc * postproc, GstCaps * caps,
     gboolean * caps_changed_ptr)
@@ -881,12 +882,10 @@ gst_vaapipostproc_update_sink_caps (GstVaapiPostproc * postproc, GstCaps * caps,
 
   GST_INFO_OBJECT (postproc, "new sink caps = %" GST_PTR_FORMAT, caps);
 
-  if (!gst_video_info_from_caps (&vi, caps))
+  if (!video_info_update (caps, &postproc->sinkpad_info, caps_changed_ptr))
     return FALSE;
 
-  if (video_info_changed (&vi, &postproc->sinkpad_info))
-    postproc->sinkpad_info = vi, *caps_changed_ptr = TRUE;
-
+  vi = postproc->sinkpad_info;
   deinterlace = is_deinterlace_enabled (postproc, &vi);
   if (deinterlace)
     postproc->flags |= GST_VAAPI_POSTPROC_FLAG_DEINTERLACE;
@@ -902,16 +901,11 @@ static gboolean
 gst_vaapipostproc_update_src_caps (GstVaapiPostproc * postproc, GstCaps * caps,
     gboolean * caps_changed_ptr)
 {
-  GstVideoInfo vi;
-
   GST_INFO_OBJECT (postproc, "new src caps = %" GST_PTR_FORMAT, caps);
 
-  if (!gst_video_info_from_caps (&vi, caps))
+  if (!video_info_update (caps, &postproc->srcpad_info, caps_changed_ptr))
     return FALSE;
 
-  if (video_info_changed (&vi, &postproc->srcpad_info))
-    postproc->srcpad_info = vi, *caps_changed_ptr = TRUE;
-
   if (postproc->format != GST_VIDEO_INFO_FORMAT (&postproc->sinkpad_info) &&
       postproc->format != DEFAULT_FORMAT)
     postproc->flags |= GST_VAAPI_POSTPROC_FLAG_FORMAT;
@@ -932,21 +926,24 @@ ensure_allowed_sinkpad_caps (GstVaapiPostproc * postproc)
     return TRUE;
 
   /* Create VA caps */
-  out_caps = gst_caps_from_string (GST_VAAPIPOSTPROC_SURFACE_CAPS ", "
+  out_caps = gst_caps_from_string (GST_VAAPI_MAKE_SURFACE_CAPS ", "
       GST_CAPS_INTERLACED_MODES);
   if (!out_caps) {
-    GST_ERROR ("failed to create VA sink caps");
+    GST_ERROR_OBJECT (postproc, "failed to create VA sink caps");
     return FALSE;
   }
 
   /* Append raw video caps */
-  if (gst_vaapipostproc_ensure_uploader (postproc)) {
-    raw_caps = GST_VAAPI_PLUGIN_BASE_UPLOADER_CAPS (postproc);
+  if (gst_vaapipostproc_ensure_display (postproc)) {
+    raw_caps =
+        gst_vaapi_plugin_base_get_allowed_raw_caps (GST_VAAPI_PLUGIN_BASE
+        (postproc));
     if (raw_caps) {
       out_caps = gst_caps_make_writable (out_caps);
       gst_caps_append (out_caps, gst_caps_copy (raw_caps));
-    } else
-      GST_WARNING ("failed to create YUV sink caps");
+    } else {
+      GST_WARNING_OBJECT (postproc, "failed to create YUV sink caps");
+    }
   }
   postproc->allowed_sinkpad_caps = out_caps;
 
@@ -1231,7 +1228,7 @@ ensure_srcpad_buffer_pool (GstVaapiPostproc * postproc, GstCaps * caps)
       GST_VIDEO_INFO_WIDTH (&vi), GST_VIDEO_INFO_HEIGHT (&vi));
 
   if (postproc->filter_pool
-      && !video_info_changed (&vi, &postproc->filter_pool_info))
+      && !video_info_changed (&postproc->filter_pool_info, &vi))
     return TRUE;
   postproc->filter_pool_info = vi;
 
@@ -1300,11 +1297,13 @@ gst_vaapipostproc_query (GstBaseTransform * trans, GstPadDirection direction,
 {
   GstVaapiPostproc *const postproc = GST_VAAPIPOSTPROC (trans);
 
-  if (gst_vaapi_reply_to_query (query,
-          GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc))) {
-    GST_DEBUG_OBJECT (postproc, "sharing display %p",
-        GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc));
-    return TRUE;
+  if (GST_QUERY_TYPE (query) == GST_QUERY_CONTEXT) {
+    if (gst_vaapi_handle_context_query (query,
+            GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc))) {
+      GST_DEBUG_OBJECT (postproc, "sharing display %p",
+          GST_VAAPI_PLUGIN_BASE_DISPLAY (postproc));
+      return TRUE;
+    }
   }
 
   return
