@@ -57,9 +57,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapisink);
 /* Default template */
 /* *INDENT-OFF* */
 static const char gst_vaapisink_sink_caps_str[] =
-    GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
-        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE,
-        "{ ENCODED, NV12, I420, YV12 }") ";"
+    GST_VAAPI_MAKE_ENC_SURFACE_CAPS ";"
 #if GST_CHECK_VERSION(1,3,1)
     GST_VIDEO_CAPS_MAKE_WITH_FEATURES (
         GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE ","
@@ -979,16 +977,6 @@ gst_vaapisink_ensure_backend (GstVaapiSink * sink)
   }
 }
 
-static gboolean
-gst_vaapisink_ensure_uploader (GstVaapiSink * sink)
-{
-  if (!gst_vaapisink_ensure_display (sink))
-    return FALSE;
-  if (!gst_vaapi_plugin_base_ensure_uploader (GST_VAAPI_PLUGIN_BASE (sink)))
-    return FALSE;
-  return TRUE;
-}
-
 static gboolean
 gst_vaapisink_ensure_render_rect (GstVaapiSink * sink, guint width,
     guint height)
@@ -1193,9 +1181,7 @@ gst_vaapisink_start (GstBaseSink * base_sink)
 {
   GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
 
-  if (!gst_vaapi_plugin_base_open (GST_VAAPI_PLUGIN_BASE (sink)))
-    return FALSE;
-  return gst_vaapisink_ensure_uploader (sink);
+  return gst_vaapi_plugin_base_open (GST_VAAPI_PLUGIN_BASE (sink));
 }
 
 static gboolean
@@ -1214,17 +1200,17 @@ static GstCaps *
 gst_vaapisink_get_caps_impl (GstBaseSink * base_sink)
 {
   GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
-  GstCaps *out_caps, *yuv_caps;
+  GstCaps *out_caps, *raw_caps;
 
   out_caps = gst_static_pad_template_get_caps (&gst_vaapisink_sink_factory);
   if (!out_caps)
     return NULL;
 
-  if (gst_vaapisink_ensure_uploader (sink)) {
-    yuv_caps = GST_VAAPI_PLUGIN_BASE_UPLOADER_CAPS (sink);
-    if (yuv_caps) {
+  if (GST_VAAPI_PLUGIN_BASE_DISPLAY (sink)) {
+    raw_caps = gst_vaapi_plugin_base_get_allowed_raw_caps (GST_VAAPI_PLUGIN_BASE (sink));
+    if (raw_caps) {
       out_caps = gst_caps_make_writable (out_caps);
-      gst_caps_append (out_caps, gst_caps_copy (yuv_caps));
+      gst_caps_append (out_caps, gst_caps_copy (raw_caps));
     }
   }
   return out_caps;
@@ -1461,15 +1447,20 @@ static gboolean
 gst_vaapisink_query (GstBaseSink * base_sink, GstQuery * query)
 {
   GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (sink);
+  gboolean ret = FALSE;
 
-  GST_INFO_OBJECT (sink, "query type %s", GST_QUERY_TYPE_NAME (query));
-
-  if (gst_vaapi_reply_to_query (query, GST_VAAPI_PLUGIN_BASE_DISPLAY (sink))) {
-    GST_DEBUG ("sharing display %p", GST_VAAPI_PLUGIN_BASE_DISPLAY (sink));
-    return TRUE;
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CONTEXT:
+      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      break;
+    default:
+      ret = GST_BASE_SINK_CLASS (gst_vaapisink_parent_class)->query (base_sink,
+          query);
+      break;
   }
-  return GST_BASE_SINK_CLASS (gst_vaapisink_parent_class)->query (base_sink,
-      query);
+
+  return ret;
 }
 
 static void
